from platform import system
from plbmng.lib import full_map
from plbmng.lib import port_scanner
from multiprocessing import Pool, Lock, Value
import subprocess
import re
import os
import webbrowser
import folium
import sqlite3
import hashlib

# global variables
base = None
increment = None
lock = None

# Constant definition
OPTION_LOCATION = 0
OPTION_IP = 1
OPTION_DNS = 2
OPTION_CONTINENT = 3
OPTION_COUNTRY = 4
OPTION_REGION = 5
OPTION_CITY = 6
OPTION_URL = 7
OPTION_NAME = 8
OPTION_LAT = 9
OPTION_LON = 10
OPTION_GCC = 11
OPTION_PYTHON = 12
OPTION_KERNEL = 13
OPTION_MEM = 14

USER_NODES = "/database/user_servers.node"
LAST_SERVER = "/database/last_server.node"
PLBMNG_DATABASE = "/database/internal.db"
PLBMNG_CONF = "/conf/plbmng.conf"
FIRST_RUN_FILE = "/database/first.boolean"
MAP_FILE = "plbmng_server_map.html"
DIALOG = None
SOURCE_PATH = None
DESTINATION_PATH = None


class NeedToFillPasswdFirstInfo(Exception):
    """Raise when password is not filled"""


class NonZeroReturnCode(Exception):
    """Raise when return code is not equal to 0"""


def get_custom_servers(start_id: str) -> list:
    """
    Read user_servers.node file in database directory a return all servers from it as list.

    :param start_id: Starting id as string.
    :type start_id: str
    :return: Return all servers from user_servers.node.
    :rtype: list
    """
    user_nodes = []
    with open(get_path() + USER_NODES) as tsv:
        lines = tsv.read().split("\n")
    for line in lines:
        if not line:
            continue
        if line[0].startswith('#'):
            continue
        columns = line.split()
        columns.insert(0, start_id)
        if len(columns) < 11:
            for column in range(11 - len(columns)):
                columns.append("unknown")
        try:
            user_nodes.append(columns)
            start_id += 1
        except ValueError:
            pass
    return user_nodes


def run_command(cmd: str) -> (int, str):
    """
    Executes given cmd param as shell command.

    :param cmd: shell command as string.
    :type cmd: str
    :return: Return exit code and standard outpur of given cmd.
    :rtype: tuple
    """
    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL, shell=True)
    try:
        stdout, stderr = process.communicate(timeout=15)
    except subprocess.TimeoutExpired:
        process.kill()
        return process.returncode, "unknown"
    stdout = stdout.decode('ascii', 'ignore')
    stdout = stdout.rstrip("\n")
    return_code = process.returncode
    return return_code, stdout


def get_servers_programs(ip_or_hostname: str, ssh=False) -> list:
    """
    Return versions of prepared commands as list.

    :param ip_or_hostname: IP address of hostname of the target
    :type ip_or_hostname: str
    :param ssh:
    :type ssh: bool
    :return: List of requested info as string
    :rtype: list
    """
    commands = ["gcc -dumpversion", "python3 --version", "uname -r",
                "grep MemTotal /proc/meminfo | awk '{print $2 / 1024}'",
                ]
    if not ssh:
        # return list of unknown string depending on number of commands
        return ["unknown" for x in range(len(commands))]
    cmd = 'ssh -o PasswordAuthentication=no -o UserKnownHostsFile=/dev/null ' \
          '-o StrictHostKeyChecking=no -o LogLevel=QUIET -o ConnectTimeout=10 ' \
          '-i %s %s@%s ' % (get_ssh_key(), get_ssh_user(), ip_or_hostname)
    output = []
    for command in commands:
        try:
            ret, stdout = run_command(cmd + command)
            if ret != 0:
                output.append("unknown")
                continue
            if stdout:
                output.append(stdout)
                continue
            else:
                output.append("unknown")
        except Exception as e:
            print(e)
            return ["unknown" for x in range(len(commands))]
    return output


def get_ssh_key() -> str:
    """
    Return path to the ssh key from plbmng conf file as string.

    return: Path to the ssh key as string.
    :rtype: str
    """
    ssh_path = ""
    with open(get_path() + PLBMNG_CONF, 'r') as config:
        for line in config:
            if re.search('SSH_KEY', line):
                ssh_path = (re.sub('SSH_KEY=', '', line)).rstrip()
    return ssh_path


def get_ssh_user() -> str:
    """
    Return slice name(remote user) from plbmng conf file as string.

    :return: Slice name(remote user) as string.
    :rtype: str
    """
    user = ""
    with open(get_path() + PLBMNG_CONF, 'r') as config:
        for line in config:
            if re.search('SLICE=', line):
                user = (re.sub('SLICE=', '', line)).rstrip()
    return user


def get_user() -> str:
    """
    Return user name from plbmng conf file as string.

    :return: User name as string.
    :rtype: str
    """
    user = ""
    with open(get_path() + PLBMNG_CONF, 'r') as config:
        for line in config:
            if re.search('USERNAME=', line):
                user = (re.sub('USERNAME=', '', line)).rstrip()
    return user


def get_passwd() -> str:
    """
    Return password from plbmng conf file as string.

    :return: Password as string.
    :rtype: str
    """
    passwd = ""
    with open(get_path() + PLBMNG_CONF, 'r') as config:
        for line in config:
            if re.search('PASSWORD=', line):
                passwd = (re.sub('PASSWORD=', '', line)).rstrip()
    return passwd


def get_all_nodes():
    """
    Get all nodes from plbmng using planetlab_list_creator script.

    :raise: NeedToFillPasswdFirstInfo
    :return: Create file default.node in plbmng database directory
    """
    user = get_user()
    passwd = get_passwd()
    if user != "" and passwd != "":
        os.system(
            "myPwd=$(pwd); cd " + get_path() + "; python3 lib/planetlab_list_creator.py "
                                               "-u \"" + user + "\" -p \"" + passwd + "\" -o ./; cd $(echo $myPwd)")
    else:
        raise NeedToFillPasswdFirstInfo


def is_first_run() -> bool:
    """
    Check first.boolean file in database directory if the user is
    using plbmng for the first time. If so, rewrites the value in the file.

    :return: True if user is using plbmng for the first time.
    :rtype: bool
    """
    is_first = get_path() + FIRST_RUN_FILE
    with open(is_first, 'r') as isFirstFile:
        bool_is_first = isFirstFile.read().strip('\n')
    if bool_is_first == "True":
        with open(is_first, 'w') as is_first_file:
            is_first_file.write("False")
        return True
    else:
        return False


def search_by_regex(nodes: list, option: int, regex: str) -> list:
    """
    Return all :param regex matched values from :param nodes at :param option index.

    :param nodes: list of nodes.
    :type nodes: list
    :param option: Index in the nodes list(check constants at the start of this file).
    :type option: str
    :param regex: Pattern to be found.
    :type regex: str
    :return: Return list of matched values as list.
    """
    answers = []
    for item in nodes:
        if re.search(regex, item[option]):
            answers.append(item[option])
    return answers


def search_by_sware_hware(nodes: list, option: int) -> dict:
    """
    Return all unique entries in :param nodes list at :param option index
    as keys and all host names which contains the entry in list as value.

    :param nodes: list of nodes.
    :type nodes: list
    :param option: Index in the nodes list(check constants at the start of this file).
    :type option: int
    :return: Return dictionary of unique entries as keys and all host names in list as value.
    :rtype: dict
    """
    filter_nodes = {}
    for item in nodes:
        if item[option] not in filter_nodes.keys():
            filter_nodes[item[option]] = [item[OPTION_DNS]]
        else:
            filter_nodes[item[option]].append(item[OPTION_DNS])
    return filter_nodes


def search_by_location(nodes: list) -> (dict, dict):
    """
    Return two dictionaries created from :param nodes list based on their continent, country and host name.\
    First dictionary contains continents(eg. EU, AS) as key and all the countries in the list(eg. ["CZ, SK"])\
    Second dictionary contains country as key(eg. "CZ") and values are all the server based in the country in list.\
    {"EU": ["CZ", "SK"...]}, {"cz": ["aaaa.cz", "bbbbb.cz"]}

    :param nodes: List of all available nodes.
    :type nodes: list
    :rtype: tuple
    """
    continents_dict = {}
    countries = {}

    for item in nodes:
        if item[OPTION_CONTINENT] in continents_dict.keys():
            continents_dict[item[OPTION_CONTINENT]].append(item[OPTION_COUNTRY])
        else:
            continents_dict[item[OPTION_CONTINENT]] = [item[OPTION_COUNTRY]]
        if item[OPTION_COUNTRY] in countries.keys():
            countries[item[OPTION_COUNTRY]].append(item[OPTION_DNS])
        else:
            countries[item[OPTION_COUNTRY]] = [item[OPTION_DNS]]
    return continents_dict, countries


def connect(mode: int, node: list):
    """
    Connect to a node using ssh or Midnight Commander.

    :param mode: If mode is equals to 1, ssh is used. If mode is equals to 2, MC is used to connect to the node.\
                 If mode is different from 1 or 2, return None.
    :type mode: int
    :param node: List which contains all information from planetlab\
    network about the node(must follow template from default.node).
    :type node: list
    :raises: ConnectionError
    """
    clear()
    key = get_ssh_key()
    user = get_ssh_user()
    if mode == 1:
        return_value = os.system(
            "ssh -o \"StrictHostKeyChecking = no\" -o \"UserKnownHostsFile=\
            /dev/null\" -i " + key + " " + user + "@" + node[OPTION_IP])
        if return_value != 0:
            raise ConnectionError("SSH failed with error code %s" % return_value)
    elif mode == 2:
        os.system('ssh-add ' + key)
        return_value = os.system("mc sh://" + user + "@" + node[OPTION_IP] + ":/home")
        if return_value != 0:
            raise ConnectionError("MC failed with error code %s" % return_value)


def show_on_map(node: list, node_info="") -> None:
    """
    Generate and open in default web browser html page with map of the world and show node on the map.

    :param node: List which contains all information from planetlab\
    network about the node(must follow template from default.node).
    :type node: list
    :param node_info: Info about node as string.
    :type node_info: str
    :rtype: None
    """
    _stderr = os.dup(2)
    os.close(2)
    _stdout = os.dup(1)
    os.close(1)
    fd = os.open(os.devnull, os.O_RDWR)
    os.dup2(fd, 2)
    os.dup2(fd, 1)

    latitude = float(node[OPTION_LAT])
    longitude = float(node[OPTION_LON])
    popup = folium.Popup(node_info["text"].strip().replace('\n', '<br>'), max_width=1000)
    node_map = folium.Map(location=[latitude, longitude],
                          zoom_start=2, min_zoom=2)
    if node_info == "":
        folium.Marker([latitude, longitude], popup=popup).add_to(node_map)
    else:
        folium.Marker([latitude, longitude], popup).add_to(node_map)
    node_map.save('/tmp/map_plbmng.html')
    try:
        webbrowser.get().open('file://' + os.path.realpath('/tmp/map_plbmng.html'))
    finally:
        os.close(fd)
        os.dup2(_stderr, 2)
        os.dup2(_stdout, 1)


def plot_servers_on_map(nodes: list, path: str) -> None:
    """
    Plot every node in nodes on map.

    :param nodes: List of Planetlab nodes.
    :type nodes: list
    :param path: path to the directory which contains MAP_FILE.
    :type path: str
    :rtype: None
    """
    _stderr = os.dup(2)
    os.close(2)
    _stdout = os.dup(1)
    os.close(1)
    fd = os.open(os.devnull, os.O_RDWR)
    os.dup2(fd, 2)
    os.dup2(fd, 1)

    # update base_data.txt file based on latest database with nodes
    full_map.plot_server_on_map(nodes)
    try:
        webbrowser.get().open('file://' + os.path.realpath(path + "/" + MAP_FILE))
    finally:
        os.close(fd)
        os.dup2(_stderr, 2)
        os.dup2(_stdout, 1)


def get_server_info(server_id: int, option: int, nodes: list) -> (dict, list):
    """
    Retrieve all available info about server from :param node
    based on :param server_id. Option should be index on which is server id present in node list.

    :param server_id: ID of the server.
    :type server_id: int
    :param option: Index program should look for server_id in node.
    :type option: int
    :param nodes: List of all nodes.
    :type nodes: list
    :return: Return dictionary with info about node and the node found in node based on server id.
    :rtype: tuple
    """
    if option == 0:
        option = OPTION_DNS
    if isinstance(server_id, str):
        # in nodes find the chosen_one node
        chosen_one = ""
        for item in nodes:
            if re.search(server_id, item[option]):
                chosen_one = item
                break
        if chosen_one == "":
            print("Internal error, please file a bug report via PyPi")
            exit(99)
        # get information about servers
        ip_or_hostname = chosen_one[OPTION_DNS] if chosen_one[OPTION_DNS] != "unknown" else chosen_one[OPTION_IP]
        info_about_node_dic = dict()
        region, city, url, fullname, lat, lon = get_info_from_node(chosen_one)
        info_about_node_dic["region"] = region
        info_about_node_dic["city"] = city
        info_about_node_dic["url"] = url
        info_about_node_dic["fullname"] = fullname
        info_about_node_dic["lat"] = lat
        info_about_node_dic["lon"] = lon
        info_about_node_dic["icmp"] = test_ping(ip_or_hostname)
        info_about_node_dic["sshAvailable"] = test_ssh(ip_or_hostname)
        programs = get_servers_programs(ip_or_hostname, info_about_node_dic["sshAvailable"])
        info_about_node_dic["text"] = """
            NODE: %s
            IP: %s
            CONTINENT: %s, COUNTRY: %s, REGION: %s, CITY: %s
            URL: %s
            FULL NAME: %s
            LATITUDE: %s, LONGITUDE: %s
            CURRENT ICMP RESPOND: %s
            CURRENT SSH AVAILABILITY: %r
            GCC version: %s Python: %s Kernel version: %s
            """ % (chosen_one[OPTION_DNS],
                   chosen_one[OPTION_IP],
                   chosen_one[OPTION_CONTINENT],
                   chosen_one[OPTION_COUNTRY],
                   info_about_node_dic["region"],
                   info_about_node_dic["city"],
                   info_about_node_dic["url"],
                   info_about_node_dic["fullname"],
                   info_about_node_dic["lat"],
                   info_about_node_dic["lon"],
                   info_about_node_dic["icmp"],
                   info_about_node_dic["sshAvailable"],
                   programs[0],
                   programs[1],
                   programs[2])
        if info_about_node_dic["sshAvailable"] is True or info_about_node_dic["sshAvailable"] is False:
            # update last server access database
            update_last_server_access(info_about_node_dic, chosen_one, get_path())
            return info_about_node_dic, chosen_one
        else:
            return False, False, False


def get_info_from_node(node: list) -> tuple:
    """
    Return basic information from node an return it as tuple

    :param node: List which contains all information from planetlab\
    network about the node(must follow template from default.node).
    :type node: list
    :return: Return region, city, url, full name, latitude and longitude from the node as tuple.
    :rtype: tuple
    """
    region = node[5]
    city = node[6]
    url = node[7]
    fullname = node[8]
    lat = node[9]
    lon = node[10]
    return region, city, url, fullname, lat, lon


"""
DEPRECATED FUNCTION
def remove_cron():
    os.system("crontab -l | grep -v \"plbmng crontab\" | crontab -")

    def add_to_cron(mode):
    if int(mode) == 1:
        line = "@daily plbmng crontab"
    elif int(mode) == 2:
        line = "@weekly plbmng crontab"
    elif int(mode) == 3:
        line = "@monthly plbmng crontab"
    os.system("echo \"$(crontab -l ; echo " + line + ")\" | crontab -")
"""


def get_path() -> str:
    """
    Return absolute path to the source directory of plbmng.

    :return: absolute path to the source directory of plbmng as str.
    :rtype: str
    """
    path = os.path.dirname(os.path.realpath(__file__)).rstrip("/lib")
    os.chdir(path)
    return path


def clear() -> None:
    """
    Clear shell.
    """
    os.system("clear")


def test_ping(target: str, return_bool=False):
    """
    Try to ping :param target host and return boolean value or message\
    based on ping command return code from ping tool.

    :param target: Host name or IP address.
    :type target: str
    :param return_bool: If set to False return message instead of boolean.
    :type return_bool: bool
    :return: Return message or bool value with ping result.
    """
    if system().lower() == 'windows':
        ping_param = '-n'
    else:
        ping_param = '-c'
    # for Linux ping parameter takes seconds while MAC OS ping takes milliseconds
    if system().lower() == 'linux':
        ping_packet_wait_time = 1
    else:
        ping_packet_wait_time = 800
    command = ['ping', ping_param, '1', target, '-W', str(ping_packet_wait_time)]
    p = subprocess.Popen(command, stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    # prepare the regular expression to get time
    if system().lower() == 'windows':
        avg = re.compile('Average = ([0-9]+)ms')
    else:
        avg = re.compile(
            'min/avg/max/[a-z]+ = [0-9.]+/([0-9.]+)/[0-9.]+/[0-9.]+')
    avg_str = avg.findall(str(p.communicate()[0]))
    if p.returncode != 0:
        if not return_bool:
            return "Not reachable via ICMP"
        return False
    else:
        p.kill()
        if not return_bool:
            return avg_str[0] + " ms"
        return True


def test_ssh(target: str):
    """
    Scan port 22 of the given :param target.

    :param target: Host name or IP address.
    :type target: str
    :return: Result of the port scanning.
    """
    result = port_scanner.test_port_availability(target, 22)
    if result is True or result is False:
        return result
    elif result == 98:
        return result
    elif result == 97:
        return result


def verify_api_credentials_exist(path: str) -> bool:
    """
    Verify that user credentials are set in the plbmng conf file.

    :param path: Path to the source directory of plbmng.
    :type path: str
    :return: Return False if USERNAME or PASSWORD is not set. If both are set, return True.
    :rtype: bool
    """
    with open(path + "/conf/plbmng.conf", 'r') as config:
        for line in config:
            if re.search('USERNAME', line):
                username = re.sub('USERNAME="(.*)"', r'\1', line).rstrip()
                if not username:
                    return False
            elif re.search('PASSWORD', line):
                password = re.sub('PASSWORD="(.*)"', r'\1', line).rstrip()
                if not password:
                    return False
    return True


def verify_ssh_credentials_exist(path: str) -> bool:
    """
    Verify that SLICE NAME(user acc on remote host) and path to SSH key are set in the plbmng conf file.

    :param path: Path to the source directory of plbmng.
    :type path: str
    :return: Return False if SLICE_NAME or SSH_KEY is not set. If both are set, return True.
    :rtype: bool
    """
    with open(path + PLBMNG_CONF, 'r') as config:
        for line in config:
            if re.search('SLICE', line):
                planetlab_slice = re.sub('SLICE="(.*)"', r'\1', line).rstrip()
                if not planetlab_slice:
                    return False
            elif re.search('SSH_KEY', line):
                key = re.sub('SSH_KEY="(.*)"', r'\1', line).rstrip()
                if not key:
                    return False
    return True


def update_last_server_access(info_about_node_dic: dict, chosen_node: list, path: str) -> None:
    """
    Update file which contains all the information about last accessed node by user.

    :param info_about_node_dic: Dictionary which contains all the info about node.
    :type info_about_node_dic: dict
    :param chosen_node: List which contains all information from planetlab\
    network about the node(must follow template from default.node).
    :type chosen_node: list
    :param path: Path to the source directory of plbmng.
    :type path: str
    """
    last_server_file = path + LAST_SERVER
    with open(last_server_file, 'w') as last_server_file:
        last_server_file.write(repr((info_about_node_dic, chosen_node)))


def get_last_server_access(path: str) -> (dict, list):
    """
    Return dictionary and list in tuple with all the available information about the last accessed node.

    :param path: Path to the source directory of plbmng.
    :type path: str
    :return: info_about_node_dic and chosen_node.
    :rtype: tuple
    """
    last_server_file = path + LAST_SERVER
    if not os.path.exists(last_server_file):
        raise FileNotFoundError
    with open(last_server_file, 'r') as last_server_file:
        info_about_node_dic, chosen_node = eval(last_server_file.read().strip('\n'))
    return info_about_node_dic, chosen_node


def server_choices(returned_choice: int, chosen_node: list, info_about_node_dic=None) -> None:
    """
    Prepared choices for user to connect or generate HTML page with map.\
    Based on the :param returned_choice, execute function with given parameters.

    :param returned_choice: Number 1-3 specified by user in DIALOG window.
    :type returned_choice: int
    :param chosen_node: List which contains all information from planetlab\
    network about the node(must follow template from default.node).
    :type chosen_node: list
    :param info_about_node_dic: Dictionary which contains all the info about node.
    :type info_about_node_dic: dict
    :rtype: None
    """
    if returned_choice is None:
        return
    elif not returned_choice:
        return
    elif int(returned_choice) == 1:
        connect(int(returned_choice), chosen_node)
    elif int(returned_choice) == 2:
        connect(int(returned_choice), chosen_node)
    elif int(returned_choice) == 3:
        show_on_map(chosen_node, info_about_node_dic)


def update_availability_database_parent(dialog, nodes=None) -> None:
    """
    Initialize parallel updating of the plbmng database.

    :param dialog: Instance of a dialog engine.
    :param nodes: List of nodes to update the database.
    :type nodes: list
    :rtype: None
    """
    global DIALOG
    increment = Value('f', 0)
    increment.value = float(100 / len(nodes))
    base = Value('f', 0)
    lock = Lock()
    DIALOG = dialog
    dialog.gauge_start()
    try:
        pool = Pool(initializer=multi_processing_init,
                    initargs=(lock, base, increment,))
    except sqlite3.OperationalError:
        dialog.msgbox("Could not update database")
    pool.map(update_availability_database, nodes)
    pool.close()
    pool.join()
    dialog.gauge_update(100, "Completed")
    dialog.gauge_stop()
    dialog.msgbox("Availability database has been successfully updated")


def multi_processing_init(l: Lock, b: Value, i: Value) -> None:
    """
    Initializer for Pool.

    :param l: Lock to synchronize processes.
    :type l: Lock
    :param b: Progress of updating the database. Value is used in DIALOG gauge.
    :type b: Value
    :param i: Incremental value in % added to :param base when process is done.
    :type i: Value
    """
    global lock
    lock = l
    global base
    base = b
    global increment
    increment = i


def update_availability_database(node: list) -> None:
    """
    Update database with given information from :param node.

    :param node: List which contains all information from planetlab\
    network about the node(must follow template from default.node).
    :type node: list
    :rtype: None
    """
    # inint block
    global PLBMNG_DATABASE, DIALOG
    db = sqlite3.connect(get_path() + PLBMNG_DATABASE)
    cursor = db.cursor()
    # action block
    ip_or_hostname = node[2] if node[2] else node[1]
    hash_object = hashlib.md5(ip_or_hostname.encode())
    ssh_result = 'T' if test_ssh(ip_or_hostname) is True else 'F'
    ping_result = 'T' if test_ping(ip_or_hostname, True) is True else 'F'
    ssh = True if ssh_result == 'T' else False
    programs = get_servers_programs(ip_or_hostname, ssh)
    # find if object exists in the database
    cursor.execute('SELECT nkey from AVAILABILITY where \
                shash = \"' + str(hash_object.hexdigest()) + '\";')
    if cursor.fetchone() is None:
        cursor.execute("INSERT into AVAILABILITY(shash, shostname, bssh, bping) VALUES\
                            (\"" + hash_object.hexdigest() + "\", \"" + ip_or_hostname + "\",\
                            \"" + ssh_result + "\", \"" + ping_result + "\")")
    else:
        cursor.execute("UPDATE availability SET bssh=\"" + ssh_result + "\", bping=\"" +
                       ping_result + "\" WHERE shash=\"" + hash_object.hexdigest() + "\"")

    cursor.execute('SELECT nkey from PROGRAMS where \
                shash = \"' + str(hash_object.hexdigest()) + '\";')
    if cursor.fetchone() is None:
        cursor.execute("INSERT into PROGRAMS(shash, shostname, "
                       "sgcc, spython, skernel, smem) VALUES\
                        (\"" + hash_object.hexdigest() + "\", \"" + ip_or_hostname + "\", \"" + programs[0] + "\",\
                                    \"" + programs[1] + "\", \"" + programs[2] + "\", \"" + programs[3] + "\")")
    else:
        cursor.execute("UPDATE programs SET sgcc=\"" + programs[0] + "\", spython=\"" +
                       programs[1] + "\", skernel=\"" + programs[2] + "\", smem=\"" + programs[
                           3] + "\" WHERE shash=\""
                       + hash_object.hexdigest() + "\"")
    # clean up

    lock.acquire()
    base.value = base.value + increment.value
    DIALOG.gauge_update(int(base.value))
    lock.release()
    db.commit()
    db.close()


def secure_copy(host: str):
    """
    Copy :param SOURCE_PATH to the :param DESTINATION_PATH to :param host. Global parameters must be set first!

    :param host: IP address or host name.
    :type host: str
    :return: Return True if command has failed. Otherwise return False.
    :rtype: bool
    """
    global SOURCE_PATH, DESTINATION_PATH
    ssh_key = get_ssh_key()
    user = get_ssh_user()
    cmd = "scp -r -o PasswordAuthentication=no -o UserKnownHostsFile=/dev/null " \
          "-o StrictHostKeyChecking=no -o LogLevel=QUIET " \
          "-i %s %s %s@%s:%s" % (ssh_key, SOURCE_PATH, user, host, DESTINATION_PATH)
    ret, stdout = run_command(cmd)
    lock.acquire()
    base.value = base.value + increment.value
    DIALOG.gauge_update(int(base.value))
    lock.release()
    if ret != 0:
        return True
    return False


def parallel_copy(dialog, source_path: str, hosts: list, destination_path: str) -> bool:
    """

    :param dialog: Instance of dialog engine.
    :param source_path: File or directory to be copied.
    :type source_path: str
    :param hosts: List of hosts(ip addressed or host names).
    :type hosts: list
    :param destination_path: Path on the target where should be file or directory copied to.
    :type destination_path: str
    :return: True if source path has been copied successfully to all hosts.
    :rtype: bool
    """
    global DIALOG, SOURCE_PATH, DESTINATION_PATH
    DIALOG = dialog
    SOURCE_PATH = source_path
    DESTINATION_PATH = destination_path
    increment = Value('f', 0)
    increment.value = float(100 / len(hosts))
    base = Value('f', 0)
    lock = Lock()
    DIALOG = dialog
    dialog.gauge_start()
    pool = Pool(initializer=multi_processing_init,
                initargs=(lock, base, increment,))
    ret = pool.map(secure_copy, hosts)
    pool.close()
    pool.join()
    dialog.gauge_update(100, "Completed")
    dialog.gauge_stop()
    # if ret is empty list, return False
    if not ret:
        return False
    # any -> If at least one scp command failed(secure_copy returned True), return False
    return not any(ret)



import folium
import sys
import os


def main():
    """
    Creates a map with location of a node.

    :return: map_one.html file
    """
    latitude = float(sys.argv[1])
    longitude = float(sys.argv[2])
    name = sys.argv[3]

    map_1 = folium.Map(location=[latitude, longitude],
                       zoom_start=2)
    folium.Marker([latitude, longitude], popup=name).add_to(map_1)

    map_1.save('map_1.html')

    sys.stdout = os.devnull
    sys.stderr = os.devnull


if __name__ == "__main__":
    main()



#! /usr/bin/env python3
# Author: Martin Kacmarcik
# License: MIT
# For my Diploma thesis at Faculty of Electrical Engineering -- Brno, University of Technology

import socket
import sys


def test_port_availability(hostname: str, port: int):
    """
    Test availability of a given port on host.

    :param hostname: Host name of a host.
    :type hostname: str
    :param port: Port number to check if is availibale.
    :type port: int
    :return: Return True if given port is available. Otherwise return False.\
    If an error has occurred, return its number.
    """
    try:
        server_ip = socket.gethostbyname(hostname)
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.settimeout(1)
        result = sock.connect_ex((server_ip, port))
        if result == 0:
            return True
        else:
            return False
    except KeyboardInterrupt:
        print("You pressed Ctrl+C")
        sys.exit(96)
    except socket.gaierror:
        return 98
    except socket.error:
        return 97
    finally:
        if "sock" in locals():
            sock.close()



import folium
import csv


def main():
    """
    Creates a map of nodes with available SSH connection.

    :return: map_ssh.html file
    """
    map_ssh = folium.Map(location=[45.523, -122.675],
                         zoom_start=2)

    with open('lib/base_data.txt') as tsv:
        for row in csv.reader(tsv, delimiter='\t'):
            name = row[0]
            try:
                x = float(row[1])
                y = float(row[2])
                print(" %s " % name)
                folium.Marker([x, y], popup=name).add_to(map_ssh)
            except ValueError:
                pass

    map_ssh.save('map_ssh.html')


if __name__ == "__main__":
    main()



#! /usr/bin/env python3
# Author: Martin Kacmarcik


import os
import sys
import signal
import locale
from dialog import Dialog
from datetime import datetime
from plbmng.lib.database import PlbmngDb
from plbmng.lib.library import update_availability_database_parent, plot_servers_on_map, \
    verify_ssh_credentials_exist, verify_api_credentials_exist, is_first_run, get_all_nodes, \
    clear, get_path, get_ssh_user, parallel_copy, OPTION_DNS, OPTION_IP, get_last_server_access, \
    server_choices, OPTION_GCC, OPTION_KERNEL, OPTION_PYTHON, OPTION_MEM, search_by_sware_hware, \
    get_server_info, search_by_location, search_by_regex, NeedToFillPasswdFirstInfo

sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)))


class Engine:
    """
    Class used for the interaction with the user and decision making based on user's input.
    """
    VERSION = "0.4.1"
    _conf_path = '/conf/plbmng.conf'
    user_nodes = '/database/user_servers.node'
    path = ""
    _debug = False

    def __init__(self):
        self.d = Dialog(dialog="dialog")
        self.db = PlbmngDb()
        locale.setlocale(locale.LC_ALL, '')
        self.d.set_background_title("Planetlab Server Manager " + self.VERSION)
        self.path = get_path()

    def _log_message(self, msg) -> None:
        """
        Write :param message to the plbmng log file.

        :param msg: Message to be written into the log file.
        :type: str
        """
        with open(self.path + "/logs/plbmng.log", "a") as log:
            log.write("[%s] INFO: %s\n" % (datetime.now(), msg))

    def init_interface(self) -> None:
        """
        Main function of Engine class. Will show root page of plbmng.
        """

        def signal_handler(sig, frame):
            clear()
            print('Terminating program. You have pressed Ctrl+C')
            exit(1)

        signal.signal(signal.SIGINT, signal_handler)
        if is_first_run():
            self.first_run_message()
        while True:
            # Main menu
            code, tag = self.d.menu("Choose one of the following options:",
                                    choices=[("1", "Access servers"),
                                             ("2", "Monitor servers"),
                                             ("3", "Plot servers on map"),
                                             ("4", "Set credentials"),
                                             ("5", "Extras")],
                                    title="MAIN MENU")

            if code == self.d.OK:
                # Access servers
                if tag == "1":
                    self.access_servers_gui()
                # Measure servers
                elif tag == "2":
                    self.monitor_servers_gui()
                # Plot servers on map
                elif tag == "3":
                    self.plot_servers_on_map_gui()
                # Set credentials
                elif tag == "4":
                    self.set_credentials_gui()
                elif tag == "5":
                    self.extras_menu()
            else:
                clear()
                exit(0)

    def extras_menu(self):
        """
        Extras menu
        """
        code, tag = self.d.menu("Choose one of the following options:",
                                choices=[("1", "Add server to database"),
                                         ("2", "Copy files to server/servers"),
                                         ("3", "Statistics"),
                                         ("4", "About"), ],
                                title="EXTRAS")
        if code == self.d.OK:
            if tag == "1":
                self.add_external_server_menu()
            elif tag == "2":
                self.copy_file()
            elif tag == "3":
                self.stats_gui(self.db.get_stats())
            elif tag == "3":
                self.about_gui(self.VERSION)

    def get_conf_path(self) -> str:
        """
        Return path to the plbmng config file.

        :rtype: str
        """
        return self._conf_path

    def set_credentials_gui(self) -> None:
        """
        Credentials Menu.
        """
        code, text = self.d.editbox(self.path + self.get_conf_path(), height=0, width=0)
        if code == self.d.OK:
            with open(self.path + self.get_conf_path(), "w") as configFile:
                configFile.write(text)

    def filtering_options_gui(self, ssh_enabled: bool, ping_enabled: bool) -> tuple:
        """
        Filtering options menu.

        :param ssh_enabled: True if ssh filtering is enabled.
        :type ssh_enabled: bool
        :param ping_enabled: True if ping filtering is enabled.
        :type ping_enabled: bool
        :return: return code from dialog engine and a tag('1' or '2').
        :rtype: tuple
        """
        code, t = self.d.checklist(
            "Press SPACE key to choose filtering options", height=0, width=0, list_height=0,
            choices=[("1", "Enable SSH accessible machines", ssh_enabled),
                     ("2", "Enable PING accessible machines", ping_enabled)], )
        return code, t

    def stats_gui(self, stats_dic: dict) -> None:
        """
        Stats menu.

        :param stats_dic: Dictionary which contains number of servers in database,\
        number of servers which responded to the ping or ssh check.
        :type stats_dic: dict
        """
        self.d.msgbox("""
        Servers in database: """ + str(stats_dic["all"]) + """
        Ssh available: """ + str(stats_dic["ssh"]) + """
        Ping available: """ + str(stats_dic["ping"]) + """
        """, width=0, height=0, title="Current statistics from last update of servers status:")

    def about_gui(self, version):
        """
        About menu.

        :param version: Current version of plbmng.
        :type version: str
        """
        self.d.msgbox("""
                PlanetLab Server Manager
                Project supervisor:
                    Dan Komosny
                Authors:
                    Tomas Andrasov
                    Filip Suba
                    Martin Kacmarcik

                Version """ + version + """
                This application is under MIT license.
                """, width=0, height=0, title="About")

    def plot_servers_on_map_gui(self):
        """
        Plot servers on map menu.
        """
        while True:
            code, tag = self.d.menu("Choose one of the following options:",
                                    choices=[("1", "Plot servers responding to ping"),
                                             ("2", "Plot ssh available servers"),
                                             ("3", "Plot all servers")],
                                    title="Map menu")
            if code == self.d.OK:
                nodes = self.db.get_nodes(True, int(tag), path=self.path)
                plot_servers_on_map(nodes, self.path)
                return
            else:
                return

    def monitor_servers_gui(self):
        """
        Monitor servers menu.
        """
        if not verify_api_credentials_exist(self.path):
            self.d.msgbox(
                "Warning! Your credentials for PlanetLab API are not set. "
                "Please use 'Set credentials' option in main menu to set them.")
        while True:
            code, tag = self.d.menu("Choose one of the following options:",
                                    # ("1", "Set crontab for status update"),
                                    choices=[("1", "Update server list now"),
                                             ("2", "Update server status now")],
                                    title="Monitoring menu", height=0, width=0)
            if code == self.d.OK:
                """if tag == "1":
                    code, tag = self.d.menu("Choose one of the following options:",
                                       choices=[("1", "Set monitoring daily"),
                                                ("2", "Set monitoring weekly"),
                                                ("3", "Set monitoring monthly"),
                                                ("4", "Remove all monitoring from cron")],
                                       title="Crontab menu")
                    if code == self.d.OK:
                        if tag == "1":
                            addToCron(tag)
                        elif tag == "2":
                            addToCron(tag)
                        elif tag == "3":
                            addToCron(tag)
                        elif tag == "4":
                            removeCron()
                    else:
                        continue"""
                if tag == "1":
                    if self.d.yesno("This is going to take around 20 minutes") == self.d.OK:
                        try:
                            get_all_nodes()
                        except NeedToFillPasswdFirstInfo:
                            self.d.msgbox(
                                "Error! Your Planetlab credentials are not set. "
                                "Please use 'Set credentials' option in main menu to set them.")
                    else:
                        continue
                elif tag == "2":
                    if self.d.yesno("This can take few minutes. Do you want to continue?") == self.d.OK:
                        if not verify_ssh_credentials_exist(self.path):
                            self.d.msgbox(
                                "Error! Your ssh credentials are not set. "
                                "Please use 'Set credentials' option in main menu to set them.")
                            continue
                        else:
                            nodes = self.db.get_nodes(path=self.path)
                            self.db.close()
                            update_availability_database_parent(dialog=self.d, nodes=nodes)
                            self.db.connect()
                    else:
                        continue
            else:
                return

    def copy_file(self):
        """
        Copy files to servers menu.
        """
        text = "Type in destination path on the target/targets."
        init = "/home/%s" % get_ssh_user()
        code, source_path = self.d.fselect(filepath="/home/", height=0, width=0)
        if code == self.d.OK:
            servers = self.access_servers_gui(checklist=True)
        else:
            return
        if not servers:
            self.d.msgbox("You did not select any servers!")
            return
        code, destination_path = self.d.inputbox(text=text, init=init)
        if code == self.d.OK:
            ret = parallel_copy(dialog=self.d, source_path=source_path,
                                hosts=servers, destination_path=destination_path)
        else:
            return
        if ret:
            self.d.msgbox("Copy successful!")
            return
        self.d.msgbox("Could not copy file/directory to the all servers!")
        return

    def access_servers_gui(self, checklist=False):
        """
        Access servers menu.

        :param checklist: True if menu shows filtered option as checkboxes.
        :type checklist: bool
        :return: If checklist is True, return all chosen servers by user.
        :rtype: list
        """
        while True:
            filter_options = self.db.get_filters_for_access_servers()
            menu_text = """
            \nActive filters: """ + filter_options

            code, tag = self.d.menu("Choose one of the following options:" + menu_text,
                                    choices=[
                                        ("1", "Filtering options"),
                                        ("2", "Access last server"),
                                        ("3", "Serach by DNS"),
                                        ("4", "Search by IP"),
                                        ("5", "Search by location"),
                                        ("6", "Search by available software/hardware")],
                                    title="ACCESS SERVERS")
            if code == self.d.OK:
                # Filtering options
                nodes = self.db.get_nodes(path=self.path)
                if tag == "1":
                    self.filtering_options_gui(self.db.get_ssh_enabled(), self.db.get_ping_enabled())
                # Access last server
                elif tag == "2":
                    self.last_server_menu()
                elif tag == "3":
                    ret = self.search_by_regex_menu(nodes, OPTION_DNS, checklist)
                    if checklist:
                        return ret
                # Search by IP
                elif tag == "4":
                    ret = self.search_by_regex_menu(nodes, OPTION_IP, checklist)
                    if checklist:
                        return ret
                # Search by location
                elif tag == "5":
                    # Grepuje se default node
                    ret = self.search_by_location_menu(checklist)
                    if checklist:
                        return ret
                elif tag == "6":
                    ret = self.advanced_filtering_menu(checklist)
                    if checklist:
                        return ret
            else:
                return

    def print_server_info(self, info_about_node_dic: dict):
        """
        Print server info menu.

        :param info_about_node_dic: Dictionary which contains all the info about node.
        :type info_about_node_dic: dict
        """
        if not verify_ssh_credentials_exist(self.path):
            prepared_choices = [("1", "Connect via SSH (Credentials not set!)"),
                                ("2", "Connect via MC (Credentials not set!)"),
                                ("3", "Show on map")]
        else:
            prepared_choices = [("1", "Connect via SSH"),
                                ("2", "Connect via MC"),
                                ("3", "Show on map")]
        code, tag = self.d.menu(
            info_about_node_dic["text"], height=0, width=0, menu_height=0, choices=prepared_choices)
        if code == self.d.OK:
            return tag
        else:
            return None

    def search_nodes_gui(self, prepared_choices, checklist=False):
        """
        Search nodes menu.

        :param prepared_choices: list of prepared choices for user.
        :type prepared_choices: list
        :param checklist: If checklist is True, crate checklist instead of menu(multiple choices).
        :type checklist: bool
        :return: Selected tag(s) from :param prepared choices.
        """
        if not prepared_choices:
            self.d.msgbox("No results found", width=0, height=0)
            return None
        while True:
            if not checklist:
                code, tag = self.d.menu("These are the results:",
                                        choices=prepared_choices,
                                        title="Search results")
            else:
                code, tag = self.d.checklist("These are the results:",
                                             choices=prepared_choices,
                                             title="Search results")
            if code == self.d.OK:
                return tag
            else:
                return None

    def first_run_message(self) -> None:
        """
        First run menu.
        """
        self.d.msgbox(
            "This is first run of the application. "
            "Please go to 'Set Credentials' menu and set your credentials now.",
            height=0, width=0)

    def need_to_fill_passwd_first_info(self):
        """
        Need to fill in password first menu.
        """
        self.d.msgbox("Credentials are not set. Please go to menu and set them now")

    def add_external_server_menu(self):
        """
        Add external server into the plbmng database(NOT TO THE PLANETLAB NETWORK!).
        """
        code, text = self.d.editbox(self.path + self.user_nodes, height=0, width=0)
        if code == self.d.OK:
            with open(self.path + self.user_nodes, "w") as nodeFile:
                nodeFile.write(text)

    def last_server_menu(self) -> None:
        """
        Return last accessed server menu.
        """
        info_about_node_dic = None
        chosen_node = None
        try:
            info_about_node_dic, chosen_node = get_last_server_access(self.path)
        except FileNotFoundError:
            self.d.msgbox("You did not access any server yet.")
        if info_about_node_dic is None or chosen_node is None:
            return
        returned_choice = self.print_server_info(info_about_node_dic)
        server_choices(returned_choice, chosen_node, info_about_node_dic)

    def advanced_filtering_menu(self, checklist: bool):
        """
        Advanced filtering menu.

        :param checklist: If checklist is True, return all chosen servers by user.
        :type checklist: bool
        """
        code, tag = self.d.menu("Filter by software/hardware:",
                                choices=[("1", "gcc version"),  # - %s" % stats["gcc"]
                                         ("2", "python version"),  # - %s" % stats["python"]
                                         ("3", "kernel version"),  # - %s" % stats["kernel"]
                                         ("4", "total memory"),  # - %s" % stats["memory"]
                                         ])
        if code == self.d.OK:
            nodes = self.db.get_nodes(choose_software_hardware=tag, path=self.path)
            answers = None
            if tag == "1":
                answers = search_by_sware_hware(nodes=nodes, option=OPTION_GCC)
            elif tag == "2":
                answers = search_by_sware_hware(nodes=nodes, option=OPTION_PYTHON)
            elif tag == "3":
                answers = search_by_sware_hware(nodes=nodes, option=OPTION_KERNEL)
            elif tag == "4":
                answers = search_by_sware_hware(nodes=nodes, option=OPTION_MEM)
            if not answers:
                return
            choices = [(item, "") for item in answers.keys()]
            returned_choice = self.search_nodes_gui(choices)
            if returned_choice is None:
                return
            hostnames = sorted(set(answers[returned_choice]))
            if not checklist:
                choices = [(hostname, "") for hostname in hostnames]
            else:
                choices = [(hostname, "", False) for hostname in hostnames]
            returned_choice = self.search_nodes_gui(choices, checklist)
            if checklist:
                return returned_choice
            if returned_choice is None:
                return
            else:
                info_about_node_dic, chosen_node = get_server_info(returned_choice, OPTION_DNS, nodes)
                returned_choice = self.print_server_info(info_about_node_dic)
            try:
                server_choices(returned_choice, chosen_node, info_about_node_dic)
            except ConnectionError as err:
                self.d.msgbox("Error while connecting. Please verify your credentials.")
                self._log_message(err)
        else:
            return

    def search_by_location_menu(self, checklist: bool):
        """
        Search by location menu.

        :param checklist: If checklist is True, return all chosen servers by user.
        :type checklist: bool
        """
        nodes = self.db.get_nodes(path=self.path)
        continents, countries = search_by_location(nodes)
        choices = [(continent, "") for continent in sorted(continents.keys())]
        returned_choice = self.search_nodes_gui(choices)
        if returned_choice is None:
            return
        choices = [(country, "") for country in countries.keys()]
        returned_choice = self.search_nodes_gui(choices)
        if returned_choice is None:
            return
        if not checklist:
            choices = [(item, "") for item in sorted(countries[returned_choice])]
        else:
            choices = [(item, "", False) for item in sorted(countries[returned_choice])]
        returned_choice = self.search_nodes_gui(choices, checklist)
        if checklist:
            return returned_choice
        if returned_choice is None:
            return
        info_about_node_dic, chosen_node = get_server_info(returned_choice, OPTION_DNS, nodes)
        returned_choice = self.print_server_info(info_about_node_dic)
        try:
            server_choices(returned_choice, chosen_node, info_about_node_dic)
        except ConnectionError as err:
            self.d.msgbox("Error while connecting. Please verify your credentials.")
            self._log_message(err)

    def search_by_regex_menu(self, nodes: list, option: int, checklist: bool):
        """
        Search by regex menu.

        :param nodes: List of all available nodes.
        :type nodes: list
        :param option: Index in the nodes list(check constants at the start of this file).
        :type option: int
        :param checklist: If checklist is True, return all chosen servers by user.
        :type checklist: bool
        """
        code, answer = self.d.inputbox("Search for:", title="Search", width=0, height=0)
        if code == self.d.OK:
            answers = search_by_regex(nodes, option=option, regex=answer)
            if not checklist:
                choices = [(item, "") for item in answers]
            else:
                choices = [(item, "", False) for item in answers]
            returned_choice = self.search_nodes_gui(choices, checklist)
            if checklist:
                return returned_choice
            if returned_choice is None:
                return
            else:
                info_about_node_dic, chosen_node = get_server_info(returned_choice, option, nodes)
                returned_choice = self.print_server_info(info_about_node_dic)
            try:
                server_choices(returned_choice=returned_choice, chosen_node=chosen_node,
                               info_about_node_dic=info_about_node_dic)
            except ConnectionError as err:
                self.d.msgbox("Error while connecting. Please verify your credentials.")
                self._log_message(err)
        else:
            return


if __name__ == "__main__":
    e = Engine()
    e.init_interface()
    exit(0)



import pandas as pd
from vincent import Visualization, Scale, DataRef, Data, PropertySet, \
    Axis, ValueRef, MarkRef, MarkProperties, Mark
import folium


def plot_server_on_map(nodes=None):
    """
    Creates a map of every known node and generates chart with information about their's latency.

    :return: map_full.html file
    """
    df = pd.DataFrame({'Data 1': [1, 2, 3, 4, 5, 6, 7, 12],
                       'Data 2': [42, 27, 52, 18, 61, 19, 62, 33]})

    # Top level Visualization
    vis = Visualization(width=500, height=300)
    vis.padding = {'top': 10, 'left': 50, 'bottom': 50, 'right': 100}

    # Data. We're going to key Data 2 on Data 1
    vis.data.append(Data.from_pandas(df, columns=['Data 2'], key_on='Data 1', name='table'))

    # Scales
    vis.scales.append(Scale(name='x', type='ordinal', range='width',
                            domain=DataRef(data='table', field="data.idx")))
    vis.scales.append(Scale(name='y', range='height', nice=True,
                            domain=DataRef(data='table', field="data.val")))

    # Axes
    vis.axes.extend([Axis(type='x', scale='x'), Axis(type='y', scale='y')])

    # Marks
    enter_props = PropertySet(x=ValueRef(scale='x', field="data.idx"),
                              y=ValueRef(scale='y', field="data.val"),
                              width=ValueRef(scale='x', band=True, offset=-1),
                              y2=ValueRef(scale='y', value=0))
    update_props = PropertySet(fill=ValueRef(value='steelblue'))
    mark = Mark(type='rect', from_=MarkRef(data='table'),
                properties=MarkProperties(enter=enter_props, update=update_props))

    vis.marks.append(mark)
    vis.axis_titles(x='days', y='latency [ms]')
    vis.to_json('vega.json')

    map_full = folium.Map(location=[45.372, -121.6972],
                          zoom_start=2)

    for node in nodes:
        if node[-2] == 'unknown' or node[-1] == 'unknown':
            continue
        x = float(node[-2])
        y = float(node[-1])
        text = """
            NODE: %s, IP: %s
            URL: %s
            FULL NAME: %s
            LATITUDE: %s, LONGITUDE: %s
            """ % (node[2],
                   node[1],
                   node[7],
                   node[8],
                   node[9],
                   node[10])
        popup = folium.Popup(text.strip().replace('\n', '<br>'), max_width=1000)
        folium.Marker([x, y], popup=popup).add_to(map_full)

    map_full.save('plbmng_server_map.html')


if __name__ == "__main__":
    plot_server_on_map()



from plbmng.lib.library import get_path, get_custom_servers
import sqlite3
import csv
import re


class PlbmngDb:
    """
    Class provides basic interaction with plbmng database.
    """
    _db_path = '/database/internal.db'
    selfbase = None
    increment = None
    base = None
    lock = None
    workers = 25  # number of workers for multiprocessing(should be equal to num of CPUs according to docs)

    def __init__(self):
        self.db = sqlite3.connect(get_path() + self._db_path)
        self.cursor = self.db.cursor()
        self.cursor.execute("select senabled from configuration where sname='ssh'")
        self.ssh_enabled = True if self.cursor.fetchone()[0] == 'T' else False
        self.cursor.execute("select senabled from configuration where sname='ping'")
        self.ping_enabled = True if self.cursor.fetchone()[0] == 'T' else False

    def connect(self) -> None:
        """
        Connect to plbmng database
        """
        self.db = sqlite3.connect(get_path() + self._db_path)
        self.cursor = self.db.cursor()

    def close(self) -> None:
        """
        Close connection to plbmng database.
        """
        self.db.close()

    def set_ssh_enabled(self, boolean: bool) -> None:
        """
        Set attribute ssh_enabled for filtering purpose.

        :param boolean: Set to True if you want to work just with server with ssh available.
        :type boolean: bool
        """
        self.ssh_enabled = boolean

    def set_ping_enabled(self, boolean: bool) -> None:
        """
        Set attribute ping_enabled for filtering purpose.

        :param boolean: Set to True if you want to work just with server which responded to the ping.
        :type boolean: bool
        """
        self.ping_enabled = boolean

    def get_ping_enabled(self) -> bool:
        """
        Return value from attribute ping_enabled.

        :rtype: bool
        """
        return self.ping_enabled

    def get_ssh_enabled(self) -> bool:
        """
        Return value from attribute ssh_enabled.

        :rtype: bool
        """
        return self.ssh_enabled

    def get_stats(self) -> dict:
        """
        Return dictionary which contains stats about ping and ssh responses.

        :rtype: dict
        """
        # Initialize filtering settings
        stat_dic = dict()

        # get numbers of all servers in database
        self.cursor.execute("select count(*) from availability;")
        stat_dic["all"] = self.cursor.fetchall()[0][0]
        # ssh available
        self.cursor.execute("select count(*) from availability where bssh='T';")
        stat_dic["ssh"] = self.cursor.fetchall()[0][0]
        # ping available
        self.cursor.execute("select count(*) from availability where bping='T';")
        stat_dic["ping"] = self.cursor.fetchall()[0][0]

        # clean up block
        return stat_dic

    def get_hw_sw_stats(self) -> dict:
        """
        Return stats how many servers responded with version of kernel, gcc, python and how many RAM server has.

        :rtype: dict
        """
        stat_dic = dict()
        self.cursor.execute("select count(*) from programs;")
        stat_dic["all"] = self.cursor.fetchall()[0][0]
        # ssh available
        self.cursor.execute("select count(*) from programs where sgcc <> 'unknown';")
        stat_dic["gcc"] = self.cursor.fetchall()[0][0]
        # ping available
        self.cursor.execute("select count(*) from programs where spython <> 'unknown';")
        stat_dic["python"] = self.cursor.fetchall()[0][0]
        self.cursor.execute("select count(*) from programs where skernel <> 'unknown';")
        stat_dic["kernel"] = self.cursor.fetchall()[0][0]
        self.cursor.execute("select count(*) from programs where smem <> 'unknown';")
        stat_dic["memory"] = self.cursor.fetchall()[0][0]
        return stat_dic

    def get_filters_for_access_servers(self):
        """
        Return message which filtering options(ssh, ping) are active.

        :rtype: str
        """
        self.cursor.execute('SELECT * from configuration;')
        configuration = self.cursor.fetchall()
        for item in configuration:
            if item[1] == 'ssh':
                if item[2] == 'T':
                    ssh_filter = True
                else:
                    ssh_filter = False
            elif item[1] == 'ping':
                if item[2] == 'T':
                    ping_filter = True
                else:
                    ping_filter = False
        if ssh_filter and ping_filter:
            filter_options = "Only SSH and ICMP available"
        elif ssh_filter and not ping_filter:
            filter_options = "Only SSH available"
        elif not ssh_filter and ping_filter:
            filter_options = "Only PING available"
        else:
            filter_options = "None"
        return filter_options

    def get_filtering_options(self) -> (bool, bool):
        """
        Return tuple which contains attribute ssh_enabled and ping_enabled.

        :rtype: tuple
        """
        return self.ssh_enabled, self.ping_enabled

    def set_filtering_options(self, tag: str) -> None:
        """
        Change filtering options based on given tag.

        :param tag: Number as string. If '1' is given, change ssh to enabled. If '2' is given, change ping to enabled.
        :type tag: str
        """
        if '1' in tag and not self.ssh_enabled:
            self.cursor.execute(
                'UPDATE configuration SET senabled="T" where sname="ssh"')
            self.db.commit()
            self.set_ssh_enabled(True)
        elif '1' not in tag and self.ssh_enabled:
            self.cursor.execute(
                'UPDATE configuration SET senabled="F" where sname="ssh"')
            self.db.commit()
            self.set_ssh_enabled(False)
        if '2' in tag and not self.ping_enabled:
            self.cursor.execute(
                'UPDATE configuration SET senabled="T" where sname="ping"')
            self.db.commit()
            self.set_ping_enabled(True)
        elif '2' not in tag and self.ping_enabled:
            self.cursor.execute(
                'UPDATE configuration SET senabled="F" where sname="ping"')
            self.db.commit()
            self.set_ping_enabled(False)

    def get_nodes(self, check_configuration=True, choose_availability_option=None,
                  choose_software_hardware=None, path="/"):
        """
        Return all nodes from default.node file plus all user specified nodes from user_servers.node.

        :param check_configuration: If set to True, check attributes ssh_enabled,\
        ping_enabled and use it as filtering option.
        :type check_configuration: bool
        :param choose_availability_option: Select filter option based on availability of ssh, ping or both.
        :type choose_availability_option: int
        :param choose_software_hardware: Select filter option from: gcc, python, kernel, mem.
        :type choose_software_hardware: str
        :param path: Path to the source directory of plbmng.
        :type: str
        :return: List of all nodes
        :rtype: list
        """
        # Initialize filtering settings
        if choose_software_hardware:
            tags = {"1": "gcc",
                    "2": "python",
                    "3": "kernel",
                    "4": "mem"}
            sql = 'SELECT * from programs where s%s not like \'unknown\'' % tags[choose_software_hardware]
        if choose_availability_option is None and choose_software_hardware is None:
            self.cursor.execute('SELECT * from configuration where senabled=\'T\';')
            configuration = self.cursor.fetchall()
            if not configuration:
                sql = 'select shostname from availability'
            else:
                sql = 'select shostname from availability where'
                for item in configuration:
                    if re.match(r'.*where$', sql):
                        sql = sql + ' b' + item[1] + '=\'T\''
                    else:
                        sql = sql + ' and b' + item[1] + '=\'T\''
        elif choose_availability_option == 1:
            sql = "select shostname from availability where bping='T'"
        elif choose_availability_option == 2:
            sql = "select shostname from availability where bssh='T'"
        elif choose_availability_option == 3:
            sql = "select shostname from availability"
        self.cursor.execute(sql)
        returned_values_sql = self.cursor.fetchall()
        # convert returned tuples to list
        server_list = {}
        for item in returned_values_sql:
            if choose_software_hardware:
                server_list[item[2]] = [x for x in item if item.index(x) >= 3]
            else:
                server_list[item[0]] = ""
        # open node file and append to the nodes if the element exists in the server_list
        node_file = path + '/database/default.node'
        nodes = []
        with open(node_file) as tsv:
            lines = csv.reader(tsv, delimiter='\t')
            for line in lines:
                if line[0] == '# ID':
                    continue
                try:
                    if check_configuration:
                        if choose_software_hardware:
                            if line[2] in server_list.keys():
                                tmp = line[:]
                                for i in server_list[line[2]]:
                                    tmp.append(i)
                                nodes.append(tmp)
                        else:
                            if line[2] in server_list.keys():
                                nodes.append(line)
                    else:
                        nodes.append(line)
                except ValueError:
                    pass
            last_id = int(line[-1][0])
        if not choose_software_hardware:
            nodes.extend(get_custom_servers(last_id))
        return nodes



from setuptools import setup, find_packages
import platform
import os

install_requires = ['vincent', 'folium', 'numpy', 'pandas', 'geocoder', 'paramiko', 'pythondialog']


def install_dialog_engine():
    """
    Try to install dialog if user is running setup for the first time. Supported OS are Fedora-like distributions.
    """
    with open("plbmng/database/first.boolean", 'r') as first:
        run = first.read().strip("\n")
    distribution = platform.platform()
    distro_check = False
    if "fc" in distribution or "el" in distribution:
        distro_check = True
    if run == "True" and distro_check:
        return_code = os.system("yum install -y dialog")
        return return_code
    return 0


ret = install_dialog_engine()
setup(name='plbmng',
      description='Tool for monitoring PlanetLab network',
      version="0.4.1.post1",
      license='MIT',
      packages=find_packages(),
      package_data={},
      include_package_data=True,
      install_requires=install_requires,
      dependency_links=['https://github.com/pandas-dev/pandas/archive/master.zip?ref=master#egg=pandas'],
      url='https://gitlab.com/utko-planetlab/plbmng',
      author='Ivan Andrasov',
      author_email='xandra03@stud.feec.vutbr.cz',
      # maintainer='Dan Komosny',
      # maintainer_email='komosny@feec.vutbr.cz'
      long_description=open("README.rst").read(),
      scripts=['bin/plbmng'],
      zip_safe=False,
      )

if ret != 0:
    print("INFO: Please make sure you have installed dialog-like engine!")
